<?php
function fetch_wp_site_data_from_URL(WP_REST_Request $request)
{
    $url = sanitize_text_field($request->get_param('url'));

    // Parse the URL to extract parts
    $parsed_url = wp_parse_url($url);
    if (!isset($parsed_url['host'])) {
        return new WP_REST_Response(['error' => 'Invalid URL provided'], 400);
    }

    // Check if the URL is the homepage
    $home_url = home_url();
    if (trailingslashit($url) === trailingslashit($home_url)) {
        // Handle homepage data
        $page_on_front = get_option('page_on_front'); // Get the page set as the homepage (if any)
        if ($page_on_front) {
            $post = get_post($page_on_front);
        } else {
            $data = [
                'post_id'   => 0,
                'title'     => get_bloginfo('name'),
                'content'   => get_bloginfo('description'),
                'post_type' => 'homepage',
                'acf'       => null,
                'permalink' => $home_url,
            ];
            return new WP_REST_Response($data, 200);
        }
    } else {
        // Retrieve the post ID from the URL
        $post_id = url_to_postid($url);
        if (!$post_id) {
            return new WP_REST_Response(['error' => 'No post found for the provided URL'], 404);
        }

        $post = get_post($post_id);
    }

    // Validate the post
    if (!$post) {
        return new WP_REST_Response(['error' => 'Post could not be retrieved'], 404);
    }

    // Prepare the response data
    $data = [
        'post_id'        => $post->ID,
        'title'          => get_the_title($post),
        'content'        => apply_filters('the_content', $post->post_content),
        'excerpt'        => get_the_excerpt($post),
        'post_type'      => $post->post_type,
        'status'         => $post->post_status,
        'author'         => get_the_author_meta('display_name', $post->post_author),
        'date'           => $post->post_date,
        'modified'       => $post->post_modified,
        'acf'            => function_exists('get_fields') ? get_fields($post->ID) : null,
        'permalink'      => get_permalink($post),
        'featured_image' => get_the_post_thumbnail_url($post, 'full'),
    ];

    return new WP_REST_Response($data, 200);
}






function get_post_data_by_url(WP_REST_Request $request)
{
    // Parse the URL to extract the path
    $url = sanitize_text_field($request->get_param('url'));
    $parsed_url = parse_url($url);
    $path = trim($parsed_url['path'], '/'); // Remove leading and trailing slashes

    // Get the post object based on the path, check for both page and post types
    $post = get_page_by_path($path, OBJECT, ['page', 'post']); // Specifically checking for 'page' and 'post'

    if (!$post) {
        return new WP_Error('not_found', 'No page or post found for this URL', array('status' => 404));
    }

    // Prepare the response data
    $data = [
        'post_id' => $post->ID,
        'title' => get_the_title($post),
        'content' => apply_filters('the_content', $post->post_content),
        'excerpt' => get_the_excerpt($post),
        'status' => $post->post_status,
        'type' => $post->post_type, // To distinguish between 'page' and 'post'
        'url' => get_permalink($post),
        'date' => $post->post_date,
        'modified' => $post->post_modified,
    ];
    ob_clean();
    return new WP_REST_Response($data, 200);
}




function fetch_wp_site_data(WP_REST_Request $request)
{
    $data = [];

    // Get all registered menus
    $menus = wp_get_nav_menus();

    // Initialize homepage_menu to null
    $data['homepage_menu'] = null;

    // Loop through each menu to check for items linking to the homepage
    foreach ($menus as $menu) {
        $menu_items = wp_get_nav_menu_items($menu->term_id);

        // Initialize a flag to check if the homepage is found in the current menu
        $homepage_found = false;

        foreach ($menu_items as $item) {
            // Check if the item URL matches the homepage URL
            if (home_url('/') === $item->url) {
                // If it matches, set homepage_menu and break the loop
                $data['homepage_menu'] = [
                    'name' => $menu->name,
                    'description' => $menu->description,
                    'items' => [], // To store menu items
                ];

                // Store menu items in the response
                foreach ($menu_items as $menu_item) {
                    $data['homepage_menu']['items'][] = [
                        'id' => $menu_item->ID,
                        'title' => $menu_item->title,
                        'url' => $menu_item->url,
                    ];
                }

                // Break out of the loop since we found the homepage menu
                $homepage_found = true;
                break; // Exit the inner loop
            }
        }

        // Break the outer loop if homepage menu has been found
        if ($homepage_found) {
            break;
        }
    }

    // Check if homepage_menu is still null and handle accordingly
    if (is_null($data['homepage_menu'])) {
        $data['homepage_menu'] = 'No menu found that links to the homepage.';
    }

    $data['list_menu'] = $menus;

    // Fetch additional data as needed
    $data['list_page'] = get_pages();
    $data['list_post'] = get_posts();
    ob_clean();
    return new WP_REST_Response($data, 200);
}



function autobloggpt_get_post(WP_REST_Request $request)
{
    $post_id = intval($request->get_param('post_id'));

    if (empty($post_id) || !get_post($post_id)) {
        return new WP_Error('invalid_post_id', 'Invalid post ID', array('status' => 400));
    }

    $post = get_post($post_id);
    $post_data = array(
        'post_id' => $post->ID,
        'title' => get_the_title($post->ID),
        'content' => apply_filters('the_content', $post->post_content),
        'date' => get_the_date('', $post->ID),
        'author' => get_the_author_meta('display_name', $post->post_author),
        'categories' => wp_get_post_categories($post->ID, array('fields' => 'names')),
        'tags' => wp_get_post_tags($post->ID, array('fields' => 'names')),
        'image_url' => get_the_post_thumbnail_url($post->ID),
    );
    ob_clean();
    return new WP_REST_Response($post_data, 200);
}


// Function to handle storing custom preview data
// function autobloggpt_preview_post(WP_REST_Request $request)
// {
//     // Extract post ID from the request
//     $post_id = intval($request->get_param('post_id'));

//     // Check if the post exists
//     $post = get_post($post_id);

//     // Extract custom title, content, and tags from the request
//     $custom_title = sanitize_text_field($request->get_param('title'));
//     $custom_content = wp_kses_post($request->get_param('content'));
//     // Extract and sanitize tags array
//     $custom_tags = $request->get_param('tags');
//     if (is_array($custom_tags)) {
//         $custom_tags = array_map('sanitize_text_field', $custom_tags); // Sanitize each tag
//     } else {
//         $custom_tags = []; // If it's not an array, fallback to empty
//     }

//     // Create a unique nonce for the preview session
//     $preview_nonce = wp_create_nonce('autobloggpt_preview_' . time());

//     // Store the custom data in a transient for 1 hour
//     $preview_data = array(
//         'title'   => $custom_title,
//         'content' => $custom_content,
//         'tags'    => $custom_tags,
//     );
//     set_transient('autobloggpt_preview_' . $preview_nonce, $preview_data, 3600); // Expires in 1 hour


//     if (!$post) {
//         $preview_url = home_url('/');
//         //return new WP_Error('invalid_post_id', 'Invalid post ID', array('status' => 400));
//     } else {
//         // Generate the native WordPress preview URL
//         $preview_url = get_preview_post_link($post_id);
//         // Append the preview_nonce to the preview URL

//     }


//     $preview_url = add_query_arg(array('preview_nonce' => $preview_nonce), $preview_url);

//     $response = wp_remote_get($preview_url);

//     $html_source = wp_remote_retrieve_body($response);
//     ob_clean();
//     // Return the preview URL
//     return new WP_REST_Response(array('preview_url' => $preview_url, 'html_source' => $html_source), 200);
// }
function autobloggpt_preview_post(WP_REST_Request $request)
{

    // Extract post ID from the request
    $post_id = $request->get_param('post_id');


    // Extract custom title, content, and tags from the request
    $custom_title = sanitize_text_field($request->get_param('title'));
    $custom_content = wp_kses_post($request->get_param('content'));
    $custom_acf_data = $request->get_param('acf') ?: []; // Fallback to empty array if no ACF data provided

    // Extract the type (post or page)
    $type = sanitize_text_field($request->get_param('type')) ?: 'post'; // Default to 'post' if not specified
    $template = sanitize_text_field($request->get_param('template')) ?: ''; // Default to empty string
    $excerpt = sanitize_text_field($request->get_param('excerpt')) ?: ''; // Default to empty string
    $event_start_date = sanitize_text_field($request->get_param('start_time')) ?: ''; // Default to empty string
    $event_end_date =  sanitize_text_field($request->get_param('end_date')) ?: ''; // Default to empty string
    // Extract and sanitize tags array
    $custom_tags = $request->get_param('tags');
    if (is_array($custom_tags)) {
        $custom_tags = array_map('sanitize_text_field', $custom_tags); // Sanitize each tag
    } else {
        $custom_tags = []; // If it's not an array, fallback to empty
    }
    //image
    $image = sanitize_text_field($request->get_param('image'));
    // Create a unique nonce for the preview session
    $preview_nonce = $request->get_param('preview_nonce') ?? '';
    $plugin_Class = 'None';
    if ($type === 'tribe_events') {
        if (!empty($preview_nonce)) {
            $preview_data = get_transient('autobloggpt_preview_' . $preview_nonce);
        }
    } else {
        $preview_nonce = wp_create_nonce('autobloggpt_preview_' . time());
        $preview_data = array(
            'title'   => $custom_title,
            'content' => $custom_content,
            'tags'    => $custom_tags,
            'acf'     => $custom_acf_data, // Include ACF data here
            'template' => $template,
            'excerpt' => $excerpt,
            'type' => $type,
            'image' => $image,
            'event_start_date' => $event_start_date,
            'event_end_date' => $event_end_date,
        );
    }

    // Case 1: post_id = 0 and no content/title, use homepage
    if ($post_id === "") {
        // Use homepage permalink for preview
        if ($template !== "") {
            // Sanitize the template filename\
            $data = get_data_template($template);
            $plugin_Class = $data['class'];
            $permalink = $data['permalink'];
        } else {
            $type = 'homepage';
            $permalink = home_url('/');
        }
    } elseif ($post_id === 0 && !empty($custom_title) && !empty($custom_content)) {
        // Check if "CMSBuddy Preview" exists
        $existing_post = get_posts(array(
            'numberposts' => 1,
            'post_type'   => $type,
            'post_status' => 'publish',
        ));


        if (!empty($existing_post)) {
            // Use the existing published post or page
            $post_id = $existing_post[0]->ID;
        } else {
            $existing_post = get_page_by_title('CMSBuddy Preview', OBJECT, $type);

            if (!$existing_post) {
                // Create a new "CMSBuddy Preview" post/page
                $post_id = wp_insert_post(array(
                    'post_title' => 'CMSBuddy Preview',
                    'post_content' => '', // Blank content initially
                    'post_status' => 'draft',
                    'post_type' => $type, // Use 'post' or 'page' based on 'type'
                ));
            } else {
                // Use the existing post/page ID
                $post_id = $existing_post->ID;
            }
        }

        $template_slug = get_page_template_slug($post_id);

        // Store custom data in a transient for 1 hour
        $preview_data['template'] = $template_slug;
        // Get permalink for the "CMSBuddy Preview" post/page

        if ($template !== "") {
            // Sanitize the template filename\
            $data = get_data_template($template);
            $plugin_Class = $data['class'];
            $permalink = $data['permalink'];
            if ($preview_nonce == '') {
                $preview_nonce = wp_create_nonce('autobloggpt_preview_' . time());
                $preview_data = array(
                    'title'   => $custom_title,
                    'content' => $custom_content,
                    'tags'    => $custom_tags,
                    'acf'     => $custom_acf_data, // Include ACF data here
                    'template' => $template,
                    'excerpt' => $excerpt,
                    'type' => $type,
                    'image' => $image,
                    'event_start_date' => $event_start_date,
                    'event_end_date' => $event_end_date,
                );
            }
        } else {
            $permalink = get_permalink($post_id);
        }
    } else {
        $post = get_post($post_id);
        $type = $post->post_type;
        if (!$post) {
            return new WP_Error('invalid_post_id', 'Invalid post ID', array('status' => 400));
        }

        // Get permalink for the existing post/page
        $permalink = get_permalink($post_id);
        $template_slug = get_page_template_slug($post_id);
        $new_preview_nonce = $request->get_param('preview_nonce');
        if ($new_preview_nonce) {
            $preview_nonce = $new_preview_nonce;
        } else {
            $preview_data['template'] = $template_slug;
        }
        // Store custom data for the preview in a transient
    }


    $preview_data['type'] = $type;
    set_transient('autobloggpt_preview_' . $preview_nonce, $preview_data, 3600); // 1-hour expiration
    // Generate the preview URL for all cases
    $preview_url = add_query_arg(array(
        // 'preview_id' => $post_id,
        'preview_nonce' => $preview_nonce,
        '_thumbnail_id' => '-1', // No thumbnail
        'preview' => 'true',
        'template' => $template,
        'plugin_Class' => $plugin_Class,
        'type' => $type,
        'post_id'=> $post_id,
    ), $permalink);

    if ($type === 'tribe_events') {
        $response = wp_remote_get($preview_url);
        $html_source = wp_remote_retrieve_body($response);
        $html_source = str_replace(
            'class="tribe-common tribe-events tribe-events-view tribe-events-view--list"',
            'class="tribe-common tribe-events tribe-events-view tribe-events-view--list tribe-common--breakpoint-xsmall tribe-common--breakpoint-medium"',
            $html_source
        );
    } else {
        if (get_post_status($post_id) === 'draft' || get_post_status($post_id) === 'pending') {

            $post = get_post($post_id);
            $html_source = apply_filters('the_content', $post->post_content);
        } else {
            // For published posts, use wp_remote_get for public URLs
            $response = wp_remote_get($preview_url);
            $html_source = wp_remote_retrieve_body($response);
            $html_source = str_replace(
                'class="tribe-common tribe-events tribe-events-view tribe-events-view--list"',
                'class="tribe-common tribe-events tribe-events-view tribe-events-view--list tribe-common--breakpoint-xsmall tribe-common--breakpoint-medium"',
                $html_source
            );

            // $html_source = apply_filters('the_content', $html_source);
        }
    }

    // // Replace the class in the MY Source HTML to include the missing classes

    // Return the preview URL and HTML source
    return new WP_REST_Response(array('preview_url' => $preview_url, 'post_id' => $post_id, 'acf' => $custom_acf_data, 'type' => $type), 200);
}




/**
 * Fetch the HTML source of a given URL.
 *
 * @param string $url The URL to fetch.
 * @return WP_REST_Response|WP_Error The HTML source or an error response.
 */
function fetch_html_source(WP_REST_Request $request)
{

    $url = sanitize_text_field($request->get_param('url'));
    // Validate the URL
    if (!filter_var($url, FILTER_VALIDATE_URL)) {
        return new WP_Error('invalid_url', 'The provided URL is invalid.', array('status' => 400));
    }

    // Use wp_remote_get to fetch the URL content
    $response = wp_remote_get($url);
    // Check for errors in the response
    if (is_wp_error($response)) {
        return new WP_Error('fetch_error', 'Error fetching the URL: ' . $response->get_error_message(), array('status' => 500));
    }

    // Get the response code
    $response_code = wp_remote_retrieve_response_code($response);

    // Check if the response code is OK (200)
    if ($response_code !== 200) {
        return new WP_Error('http_error', 'Failed to fetch content. HTTP Response Code: ' . $response_code, array('status' => $response_code));
    }

    // Retrieve the body of the response
    $html_source = wp_remote_retrieve_body($response);
    ob_clean();
    // Return the HTML source as a REST response
    return new WP_REST_Response(array('html_source' => $html_source), 200);
}



add_action('pre_get_posts', function ($query) {
    if (!is_admin() && $query->is_main_query() && isset($_GET['preview_nonce'])) {
        $query->set('post_status', ['publish', 'draft']);
    }
});


add_filter('the_content', function ($content) {
    if (isset($_GET['preview_nonce'])) {
        $preview_nonce = sanitize_text_field($_GET['preview_nonce']);
        $preview_data  = get_transient('autobloggpt_preview_' . $preview_nonce);
        if ($preview_data && !empty($preview_data['content'])) {
            // Replace the content with your preview content
            return $preview_data['content'];
        }
    }

    return $content; // Return the original content if no preview data
});





// add_action('template_include', function ($template) {
//     if (isset($_GET['preview_nonce'])) {
//         // Extract the preview_nonce
//         $preview_nonce = sanitize_text_field($_GET['preview_nonce']);
//         $post_id = $_GET['post_id'];
//         // Retrieve the associated preview data
//         $preview_data = get_transient('autobloggpt_preview_' . $preview_nonce);

//         if ($preview_data) {

//             // Get the post object
//             $post = get_post($post_id);

//             if ($post) {
//                 // Set global $post for proper behavior
//                 global $post;
//                 $post = get_post($post_id);
//                 setup_postdata($post);

//                 $template = get_page_template_slug($post_id) ?: locate_template('page.php');

//                 // Reset global $post
//                 wp_reset_postdata();

//                 return $template;
//             } else {
//                 // If post doesn't exist
//                 wp_die('Invalid post ID in preview data.');
//             }
//         } else {
//             // If no valid preview data
//             wp_die('Invalid preview nonce or preview expired.');
//         }
//     }

//     return $template;
// });
